package com.odhen.taa;

import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.net.Uri;
import android.os.Bundle;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;

public class MainActivity extends AppCompatActivity {

    private WebView webView;

    public static final int REQUEST_SELECT_FILE = 100;
    public ValueCallback<Uri[]> uploadMessage;

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        if (requestCode == REQUEST_SELECT_FILE) {
            if (uploadMessage == null) return;
            uploadMessage.onReceiveValue(WebChromeClient.FileChooserParams.parseResult(resultCode, data));
            uploadMessage = null;
        }
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

        webView = (WebView) findViewById(R.id.taaWebView);
        WebSettings webSettings = webView.getSettings();
        webSettings.setJavaScriptEnabled(true);
        webView.setWebViewClient(new Callback());
        webView.setWebContentsDebuggingEnabled(true);
        webSettings.setDomStorageEnabled(true);
        webSettings.setDatabaseEnabled(true); // enable IndexedDB

//        webSettings.setAllowFileAccessFromFileURLs(true);
//        webSettings.setAllowUniversalAccessFromFileURLs(true);
//        webSettings.setAllowFileAccess(true);
//        webSettings.setAllowContentAccess(true);

        webView.setWebViewClient(new WebViewClient() {
            @Override
            public void onPageFinished(WebView view, String url) {
                // Execute JavaScript code after the page has finished loading
                webView.evaluateJavascript("localStorage.setItem('runningEnvironment', JSON.stringify({insideAndroid: true}))", null);
            }
        });
        webView.setWebChromeClient(new MyWebChromeClient());

        webView.addJavascriptInterface(new SitefIntegration(this, webView), "Android");
        webView.addJavascriptInterface(new PrinterIntegration(this), "PrinterIntegration");

        webView.loadUrl("http://192.168.100.144:8080/");
        // webView.loadUrl("file:///android_asset/www/index.html");
    }

public class MyWebChromeClient extends WebChromeClient {
    // reference to activity instance. May be unnecessary if your web chrome client is member class.

    public boolean onShowFileChooser(WebView webView, ValueCallback<Uri[]> filePathCallback, FileChooserParams fileChooserParams) {
        // make sure there is no existing message
        if (MainActivity.this.uploadMessage != null) {
            MainActivity.this.uploadMessage.onReceiveValue(null);
            MainActivity.this.uploadMessage = null;
        }

        MainActivity.this.uploadMessage = filePathCallback;

        Intent intent = fileChooserParams.createIntent();
        try {
            MainActivity.this.startActivityForResult(intent, MainActivity.this.REQUEST_SELECT_FILE);
        } catch (ActivityNotFoundException e) {
            MainActivity.this.uploadMessage = null;
            Toast.makeText(MainActivity.this, "Cannot open file chooser", Toast.LENGTH_LONG).show();
            return false;
        }

        return true;
    }
}

private class Callback extends WebViewClient {

    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) {
        return false;
    }
}

    @Override
    public void onBackPressed() {
        if (webView != null && webView.canGoBack()) {
            webView.goBack();
        } else {
            super.onBackPressed();
        }
    }
}
