package com.odhen.taa;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.webkit.JavascriptInterface;
import android.content.Context;

import org.json.JSONException;
import org.json.JSONObject;

import br.com.softwareexpress.sitef.android.CliSiTef;
import br.com.softwareexpress.sitef.android.CliSiTefI;
import br.com.softwareexpress.sitef.android.ICliSiTefListener;

import android.webkit.WebView;
import android.widget.Toast;

public class SitefIntegration extends Activity implements ICliSiTefListener {
    Context mContext;
    private String TAG = "|SitefIntegration| ";
    private CliSiTef sitef;
    private SitefIntegration me;
    private WebView webView;

    protected PaymentData paymentData;
    private JSONObject dataObject = null;
    private JSONObject returnObject = null;
    private String givenCommands = "";
    private String messageToSend = "";

    /** Instantiate the interface and set the context */
    // Add a constructor that receives the webView object
    SitefIntegration(Context c, WebView webView) {
        mContext = c;
        this.webView = webView;
    }

    // Add a setter method to set the webView object after the SitefIntegration object is created
    public void setWebView(WebView webView) {
        this.webView = webView;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
    }

    @JavascriptInterface
    public void showToast(String message) {
        Toast.makeText(mContext, message, Toast.LENGTH_SHORT).show();
    }

    private JSONObject jsonEncode(String data) throws JSONException {
        return new JSONObject(data);
    }

    private void initSitefInstanceAndMe() {
        sitef = new CliSiTef(mContext.getApplicationContext());
        sitef.setActivity((Activity) mContext);
        me = this;
    }

    private void closeSitefInstanceAndMe() {
        if(sitef != null){
            sitef.setActivity(null);
        }
        sitef = null;
        me = null;
    }

    @JavascriptInterface
    public String configure (String data) {
        String retorno = null;
        try {
            dataObject = jsonEncode(data);
            this.paymentData = new PaymentData(dataObject);

            returnObject = new JSONObject();
            returnObject.put("error", true);
            returnObject.put("message", "");
            returnObject.put("errorCode", null);

            initSitefInstanceAndMe();

            int sts = sitef.configure(paymentData.getPaymentIp(), paymentData.getPaymentStore(), paymentData.getPaymentTerminal(), paymentData.getParams());

            if (sts == 0) {
                retorno = "{\"error\": false, \"result\": " + sts + "}";
                return retorno;
            } else {
                String message = new ErrorMessages().getErrorMessageFromStatus(sts);
                retorno = "{\"error\": true, \"result\": " + sts + ", \"message\": \"" + message + "\"}";
                return retorno;
            }

        }catch (JSONException err){
            Log.d(TAG + "ConfigureError", err.toString());
            return err.toString();
        }
    }

    @JavascriptInterface
    public String startTransaction (String data) {
        String retorno = null;

        try {
            try {
                dataObject = jsonEncode(data);
                paymentData.updatePaymentDataWithStart(dataObject);
            }catch (JSONException err){
                Log.d(TAG + "startTransaction ERROR", err.toString());
                return err.toString();
            }

            String restricoes = "{DevolveStringQRCode=1}";
            int sts = sitef.startTransaction(
                    this, paymentData.getPaymentType(), paymentData.getPaymentValue(),
                    paymentData.getPaymentInvoice(), paymentData.getPaymentDate(),
                    paymentData.getPaymentHour(), paymentData.getPaymentOperator(),
                    restricoes
            );

            if (sts != 10000) {
                retorno = "Inicia - ERRO: " + new ErrorMessages().getErrorMessageFromStatus(sts);
            } else {
                boolean canReturn = false;
                do {
                    if (messageToSend != "") {

                        JSONObject result = new JSONObject("{"
                                + "'retornoDll': " + sts
                                + ", 'funcao': " + paymentData.getPaymentType()
                                + ", 'valor': '" + paymentData.getPaymentValue() + "'"
                                + ", 'cupomFiscal': '" + paymentData.getPaymentInvoice() + "'"
                                + ", 'dataFiscal': '" + paymentData.getPaymentDate() + "'"
                                + ", 'horario': '" + paymentData.getPaymentHour() + "'"
                                + ", 'operador': '" + paymentData.getPaymentOperator() + "'"
                                + "}");

                        JSONObject json = new JSONObject("{"
                                + "'error': false, "
                                + "'result': '" + result + "'"
                                + ", 'message': '" + messageToSend + "'}");

                        retorno = json.toString();
                        messageToSend = "";
                        canReturn = true;
                    } else {
                        Thread.sleep(200);
                    }
                } while (!canReturn);
            }

        } catch (Exception err) {
            retorno = err.toString();
            Log.d(TAG + "startTransaction ERROR", retorno);
        }

        return retorno;
    }

    @Override
    public void onData(int stage, int command, int fieldId, int minLength, int maxLength, byte [] input) {
        try {

            JSONObject data = new JSONObject("{"
                    + "'type' : 'onData'"
                    + ", 'stage' :" + stage
                    + ", 'command' :" + command
                    + ", 'fieldId' :" + fieldId
                    + ", 'minLength' :" + minLength
                    + ", 'maxLength' :" + maxLength
                    + ", 'input' : '" + input.toString() + "'"
                    + ", 'buffer' :'" + sitef.getBuffer() + "'"
                    + "}");

            messageToSend = data.toString();

        } catch (Exception e) {
            Log.d(TAG, e.getMessage().toString());
            sitef.abortTransaction(0);
        }
    }

    @Override
    public void onTransactionResult(int currentStage, int resultCode) {
        try {

            JSONObject data = new JSONObject("{"
                + "'type' : 'onTransactionResult'"
                + ", 'stage' :" + currentStage
                + ", 'resultCode' :" + resultCode
                + "}");

            messageToSend = data.toString();

        } catch (Exception e) {
            Log.d(TAG, e.getMessage());
            sitef.abortTransaction(0);
        }
    }

    @JavascriptInterface
    public String continueTransaction(String buffer) {

        try {
            int code = sitef.continueTransaction(buffer);

            if (code == 0) {
                boolean canReturn = false;
                String retorno = "";

                do {
                    if (messageToSend != "") {
                        JSONObject data = new JSONObject("{'error': false, 'result': " + code + ", 'message': " + messageToSend + "}");
                        retorno = data.toString();
                        messageToSend = "";
                        canReturn = true;
                    } else {
                        Thread.sleep(200);
                    }
                } while (!canReturn);

                return retorno;
            } else {
                return "{\"error\": false, \"result\": " + code + "}";
            }
        } catch (Exception e) {
            return "{\"error\": true}";
        }
    }

    @JavascriptInterface
    public String finishTransaction (int confirma, String cupomFiscal, String dataFiscal,
                                     String horario, String parametrosAdicionais) {
        try {
            int result = sitef.finishTransaction(confirma, cupomFiscal, dataFiscal, horario, parametrosAdicionais);

            if (result == 10000) {
                boolean canReturn = false;
                String retorno = "";

                do {
                    if (messageToSend != "") {
                        JSONObject data = new JSONObject("{'error': false, 'result': " + result + ", 'message': " + messageToSend + "}");
                        retorno = data.toString();
                        messageToSend = "";
                        canReturn = true;
                    } else {
                        Thread.sleep(200);
                    }
                } while (!canReturn);

                return retorno;
            } else {
                return "{\"error\": false, \"result\": " + result + "}";
            }
        } catch (Exception e) {
            return  "{\"error\": true}";
        }
    }

    @JavascriptInterface
    public String abortTransaction (int continua) {

        try {
            int result = sitef.abortTransaction(continua);
            return "{\"error\": false, \"result\": " + result + "}";
        } catch (Exception e) {
            return  "{\"error\": true}";
        }
    }
}
